#!/usr/bin/env python
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : MkJob.py
#   Created Time  : 2019-10-30 18:28
#   Last Modified : 2019-10-30 18:28
#   Describe      :
#
# ====================================================
import os

from ShortJob import MkMulJob


class MkJob(MkMulJob.MkMulJob):
    """ Make job with only one task, inherited from MkMulJob
    """
    def __init__(self, cpp="fit.cxx"):
        MkMulJob.MkMulJob.__init__(self, [cpp])
        self._cwd = os.getcwd()

    def Make(self, index, var):
        """generate a bash script contains only one task
        Args:
            index(int): the ID of job, the job is named as job_{index}.sh
            var(tuple or list): the in put args for the task
        Returns
            void
        """
        tail = self._cpp.split('.')[0]
        logName = "log_{}.{}".format(index, tail)
        MkMulJob.Make(index, [var], [logName])



#!/usr/bin/env python
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : MkMulJob.py
#   Created Time  : 2019-10-30 18:29
#   Last Modified : 2019-12-30 20:16
#   Describe      :
#
# ====================================================
import os
from ShortJob import command


class MkMulJob(object):
    """generate a bash script contains only more than one task
    Example: the task is a list
    tasks = ["buy.csh", "mix.py", "cock.cxx", "sell.sh"]
    and the args for each task is
    [(0.1, 2, 0.3, 4), ("a", "b"), (2, "cack"), (3.2)]
    the recommend style is
    mkjob = MkMulJob(tasks)
    mkjob.Make(0, [(0.1, 2, 0.3, 4), ("a", "b"), (2, "cack"), (3.2)])
    """
    def __init__(self, cppList=["fit.cxx"]):
        self._cppList = cppList
        self._cwd = os.getcwd()

    def Make(self, index, varList=[], logName=[]):
        """generate a bash script contains only one task
        Args:
            index(int): the ID of job, the job is named as job_{index}.sh
            varList(tuple or list): each element is refered as an input args
            for one task
            logName(list): give each log a special name
        Returns
            void
        """
        if len(logName) != len(self._cppList) and len(logName) > 0:
            print("check the length")
            return
        if len(logName) == 0:
            for i in self._cppList:
                logName.append(i.split('.')[0])

        f = open("job_%d.sh" % (index), 'w')
        f.write("cd %s\n" % (self._cwd))
        f.write("cd .. \n")
        for i in range(len(varList)):
            f.write(
                self._GetCommands(index, self._cppList[i], varList[i],
                                  logName[i]) + "\n")
        f.close()

    def _GetCommands(self, index, cpp, var=[], logName="txt"):
        fileType = cpp.split('.')[-1]
        log = "log" + str(index) + "." + logName
        if fileType in command.rootType and fileType != "":
            if len(var) == 0:
                commands = "{exe} {file} > {jobpath}/{Log}".format(
                    exe=command.exe[fileType],
                    file=cpp,
                    jobpath=self._cwd.split("/")[-1],
                    Log=log)
                return commands
            else:
                commands = "{exe} '{file}({Var})' > {jobpath}/{Log}".format(
                    exe=command.exe[fileType],
                    file=cpp,
                    jobpath=self._cwd.split("/")[-1],
                    Log=log,
                    Var=",".join(self._getVarList(var)))
                return commands
        commands = "{exe} {file} {Var} > {jobpath}/{Log}".format(
            exe=command.exe[fileType],
            file=cpp,
            jobpath=self._cwd.split("/")[-1],
            Log=log,
            Var=" ".join(var))
        return commands

    def _getVarList(self, varList):
        vList = []
        for i in varList:
            if isinstance(i, int) or isinstance(i, float):
                vList.append(str(i))
            else:
                vList.append('"{}"'.format(i))
        return vList



#!/usr/bin/env python3
# encoding: utf-8
# coding style: pep8
# ====================================================
#   Copyright (C)2020 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : command.py
#   Create Time   : 2020-03-14 14:50
#   Last Modified : 2020-03-14 14:50
#   Describe      :
#
# ====================================================
import os
import ConfigParser

"""
dir: default, the default execute commands associated with one special type.
@key: file type, "C", "cxx", "cc", "cpp", "c++", "py", "sh", "csh"
@value: the execute commands, "root -l -b -q", "bash"
list: rootType
You can modify the command free in the file ~/.ShortJob/command
we define those type file belong to "ROOT"
["cxx", "C", "cc", "c++", "c"]
* in python3 the module "ConfigParser" is named as "configparser"
"""


class MyConfigParser(ConfigParser.ConfigParser):
    """set ConfigParser options for case sensitive."""
    def __init__(self, defaults=None):
        ConfigParser.ConfigParser.__init__(self, defaults=defaults)

    def optionxform(self, optionstr):
        return optionstr


default = {
    "cxx": "root -l -b -q",
    "C": "root -l -b -q",
    "c": "root -l -b -q",
    "cc": "root -l -b -q",
    "cpp": "root -l -b -q",
    "c++": "root -l -b -q",
    "py": "python",
    "sh": "bash",
    "csh": "tcsh",
}
localConfigFile = os.path.expanduser("~/.ShortJob/command")


def initConfig(execommand):
    """init the ConfigParser file, the commands is stored in ~/.ShortJob/command
    Args:
       execommand(dir): take the file type as key and the command as value
    Returns:
        void
    """
    local_config = MyConfigParser()
    local_config.add_section('command')
    for k in execommand:
        local_config.set('command', k, execommand[k])
    if not os.path.exists(os.path.expanduser("~/.ShortJob")):
        os.mkdir(os.path.expanduser("~/.ShortJob"))
    localFile = os.path.expanduser(localConfigFile)
    with open(localFile, 'w') as configFile:
        local_config.write(configFile)


def init(configfile):
    if not os.path.exists(configfile):
        initConfig(default)


def ReadConfig(configfile):
    init(configfile)
    local_config = MyConfigParser()
    local_config.read(configfile)
    d = {}
    for item in local_config.items("command"):
        d[item[0]] = item[1]
    return d


exe = ReadConfig(localConfigFile)
rootType = ["cxx", "C", "cc", "c++", "c"]

if __name__ == "__main__":
    for i in exe:
        print("{} --> {}".format(i, exe[i]))



from ROOT import TTree, TFile
from array import array


class GetResult:
    """
    Reqiure ROOT
    How:
        The program tries to find any pattern "name value" in the log file
        For example:
            alpha 0.321
        In this case, the "alpha" is assigned to be 0.021.
            beta 0.01 0.321 +/- 0.001
        In this case, the "beta" is assigned to be 0.321 +/- 0.001, while
        0.001 is the uncertainty
        * Warrn: the new value will overwrite the old one.
    Example:
        info = GetResult.("fit.log")
        fcn = info.FCN()
        tmpArgValue = info.GetVal("alpha")
    """
    def __init__(self, log="log0000"):
        self._log = log

    def SetLog(self, log):
        """Set the log file
        Args:
        log(str): the name of log file
        Returns:
          void
        """
        self._log = log

    def FCN(self):
        """obtain the FCN of likelihood fit, FCN = -log(L)
        Args:
          none
        Returns:
          FCN(double): the minimum -log(L)
        """
        for line in open(self._log):
            if "RooFitResult:" not in line:
                continue
            ll = line.split()
            ii = ll.index("value:")
            return float(ll[ii + 1].split(",")[0])

    def GetVal(self, var="sigma"):
        """ get the value of "var" from the log file
        Args:
           var(str): the name of variable
        Returns:
           value(double): the value of `var`
        """
        ll = []
        ll2 = []
        for line in open(self._log):
            if len(line.split()) == 0:
                continue
            if line.split()[0] == var and r"+/-" in line:
                # print line
                ll = line.split()
            if line.split()[0] == var and len(line.split()) == 2:
                ll2 = line.split()
                # break
        if len(ll) != 0:
            ii = ll.index(r"+/-")
            return float(ll[ii - 1])
        if len(ll2) != 0:
            return float(ll2[-1])
        return self.FF(var)
        return 0.0

    def GetError(self, var="sigma"):
        """ get the value of uncertainty of "var" from the log file
        Args:
           var(str): the name of variable
        Returns:
           value(double): the uncertainty of `var`
        """
        ll = []
        ll2 = []
        for line in open(self._log):
            if len(line.split()) == 0:
                continue
            if line.split()[0] == var and r"+/-" in line:
                # print line
                ll = line.split()
            if line.split()[0] == var and len(line.split()) == 2:
                ll2 = line.split()
                # break
        if len(ll) != 0:
            ii = ll.index(r"+/-")
            return float(ll[ii + 1])
        if len(ll2) != 0:
            return float(ll2[-1])
        return self.FF(var)
        return 0.0

    def WriteToFile(self, inputval=[], output="out.root"):
        """write a list to root file, the tree name is default as "sig",
         the branch is default as "val"
        Args:
            inputval(list): the input list, all element will convert into
              double
            output(str): the name of root file, the default value is "out.root"
        Returns:
            void
        """
        fout = TFile(output, "recreate")
        tout = TTree("sig", "")
        val = array("d", [0.0])
        tout.Branch("val", val, "val/D")
        for i in inputval:
            val[0] = i
            tout.Fill()
        tout.Write()
        fout.Close()



#!/usr/bin/env python
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : setup.py
#   Created Time  : 2019-09-24 10:51
#   Last Modified : 2019-09-29 12:09
#   Describe      :
#
# ====================================================

from setuptools import setup
from setuptools import find_packages
import sys
import os
m_version = "1.7"
disrip=open("README.md").read()
if sys.argv[1] == "publish":
    os.system("python setup.py sdist")
    os.system("python setup.py bdist_wheel")
    os.system("twine upload dist/*{}* --skip-existing".format(m_version))
else:
    setup(name='ShortJob',
      version=m_version,
      author='Xin-Xin Ma',
      packages=find_packages(),
      description="Multitask Multilanguage",
      project_urls={
          'Source': 'https://github.com/xxmawhu/ShortJob',
      },
      )



"""
This package is consistent of four module, i.e. `command`, `MkJob`, `MkMulJob`,
`GetResult`.
* command Specify each fileType with one execute command, such as
> .sh -> bash
> .C -> root -l -b -q
* MkMulJob generate a massive bash scripts with input tasks
Example:
```bash
cd /home/maxx/work/GitHub/ShortJob/test
cd ..
root -l -b -q 'run.C(3.14,2.3,"a")' > test/log1.run
python test.py a b c > test/log1.test
bash run.sh a > test/log1.run
```
* MkJob generate a massive bash scripts with only one task
* GetResult pick the result of interest from the log file.

"""
from ShortJob import MkJob
from ShortJob import MkMulJob
from ShortJob import command
